#include "afflib.h"
#include "afflib_i.h"
#include "aimage.h"

#include "imager.h"


void checkline(char *name,char *segname,
	       char *buf,char *copy,AFFILE *af,int number)
{
    char *cc;
    while(buf[0] && isspace(buf[0])) buf++; // advance buf to end of spaces

    if(strncmp(buf,name,strlen(name))!=0) return;
    cc = buf+strlen(name);		// get a pointer
    while(*cc && isspace(*cc)) cc++;	// scan to end of spaces
    char *dd = index(cc,'\n');		// can we find a \n?
    if(dd) *dd = '\000';		// yes; clear it
    if(number){
	af_write_seg(af,segname,atoi(cc),0,0); // store it in the argument...
    }
    else {
	af_write_seg(af,segname,0,cc,strlen(cc)); // store it in the data area
	if(copy) strcpy(copy,cc);		// make a copy
    }
}




/* FreeBSD-specific ident routines */
#ifdef __FreeBSD__
#include <sys/param.h>
#include <sys/mount.h>

int freebsd_get_params(const char *infile,int *controller,int *channel)
{
  int adnum;
  if(sscanf(infile,"/dev/ad%d",&adnum)==1){
    if(controller) *controller = adnum / 2;
    if(channel)    *channel    = adnum % 2;
    return 0;
  }
  return(-1);
}

#define IDENT_DEFINED
/* ident for freebsd */
void ident(const char *infile,AFFILE *af)
{
  int controller = 0;
  int channel    = 0;
  
  /* Check for a SCSI drive */
  if(strncmp(infile,"/dev/da",7)==0){
     FILE *f = popen("camcontrol inquiry 2:0","r");
     char capbuf[65536];
     char buf[256];
     capbuf[0] = 0;

     while(!feof(f)){
       buf[0] = 0;
       fgets(buf,sizeof(buf)-1,f);
       strcat(capbuf,buf);
       char *cc = index(buf,'>'); // see if it should be terminated
       if(cc) *cc = 0;
       checkline("pass2: <",AF_DEVICE_MODEL,buf,device_model,af,0);
       checkline("Serial Number",AF_DEVICE_SN,buf,serial_number,af,0);
     }
     pclose(f);
     af_update_seg(af,AF_DEVICE_CAPABILITIES,0,capbuf,strlen(capbuf),1);
     return;
  }
				     


  /* Check for an IDE drive */
  if(strncmp(infile,"/dev/ad",7)==0 &&
     freebsd_get_params(infile,&controller,&channel)==0){
	char capbuf[65536];
	char buf[256];
#if __FreeBSD_version > 600000
	/* Syntax of command was changed in FreeBSD 6.0 */
	sprintf(buf,"atacontrol cap ad%d",controller*2+channel);
#else
	sprintf(buf,"atacontrol cap ata%d %d",controller,channel);
#endif
	puts(buf);
	capbuf[0] = 0;
	FILE *f = popen(buf,"r");
	while(!feof(f)){
	    buf[0] = 0;
	    buf[255] = 0;
	    fgets(buf,sizeof(buf)-1,f);
	    strcat(capbuf,buf);
	    checkline("device model",AF_DEVICE_MODEL,buf,device_model,af,0);
	    checkline("serial number",AF_DEVICE_SN,buf,serial_number,af,0);
	    checkline("firmware revision",AF_DEVICE_FIRMWARE,
		      buf,firmware_revision,af,0);
	    checkline("cylinders",AF_CYLINDERS,buf,0,af,1);
	    checkline("heads",AF_HEADS,buf,0,af,1);
	    checkline("sectors/track",AF_SECTORS_PER_TRACK,buf,0,af,1);
	}
	pclose(f);
	af_update_seg(af,AF_DEVICE_CAPABILITIES,0,capbuf,strlen(capbuf),1);
	return;
  }
}


#define MAKE_ATTACH_COMMANDS_DEFINED
void make_attach_commands(char *attach,char *detach,char *master,char *slave,int dev)
{
    sprintf(attach,"atacontrol attach ata%d",dev);
    sprintf(detach,"atacontrol detach ata%d",dev);
    sprintf(master,"/dev/ad%d",dev*2);
    sprintf(slave,"/dev/ad%d",dev*2+1);
}

#endif





#ifdef linux
#define IDENT_DEFINED
void ident(const char *infile,AFFILE *af)
{
  if(access(infile,R_OK)==0){
      char capbuf[65536];
      char buf[256];
      sprintf(buf,"hdparm -I %s",infile);
      puts(buf);
      capbuf[0] = 0;
      FILE *f = popen(buf,"r");
      while(!feof(f)){
	  buf[0] = 0;
	  buf[255] = 0;
	  fgets(buf,sizeof(buf)-1,f);
	  strcat(capbuf,buf);
	  checkline("Model Number:",AF_DEVICE_MODEL,buf,device_model,af,0);
	  checkline("Serial Number:",AF_DEVICE_SN,buf,serial_number,af,0);
	  checkline("Firmware Revision:",AF_DEVICE_FIRMWARE,
		    buf,firmware_revision,af,0);
	  checkline("cylinders",AF_CYLINDERS,buf,0,af,1);
	  checkline("heads",AF_HEADS,buf,0,af,1);
	  checkline("sectors/track",AF_SECTORS_PER_TRACK,buf,0,af,1);
      }
      pclose(f);
      af_put_seg(af,AF_DEVICE_CAPABILITIES,0,capbuf,strlen(capbuf));
  }
}


/*
 * Here are some parameters you may find interesting:
 *       hdparm -R 0x1700 0 0 /dev/hda   (to remove /dev/hda)
 *       ide2: BM-DMA at 0xb800-0xb807, BIOS settings: hde:DMA, hdf:pio
 */
#define MAKE_ATTACH_COMMANDS_DEFINED
void make_attach_commands(char *attach,char *detach,char *master,char *slave,int dev)
{
    system("/sbin/mknod /tmp/hda c  3 0");
    system("/sbin/mknod /tmp/hdb c  3 64");
    system("/sbin/mknod /tmp/hdc c 22 0");
    system("/sbin/mknod /tmp/hdd c 22 64");
    sprintf(attach,"hdparm -R %d",dev);
    sprintf(detach,"hdparm -U %d",dev);
    sprintf(master,"/dev/ad%d",dev*2);
    sprintf(slave,"/dev/ad%d",dev*2+1);
}
#endif


#ifndef IDENT_DEFINED
void ident(const char *infile,AFFILE *af)
{
}
#endif



#ifndef MAKE_ATTACH_COMMANDS_DEFINED
void make_attach_commands(char *attach,char *detach,char *master,char *slave,int dev)
{
    attach[0] = 0;
    detach[0] = 0;
    master[0] = 0;
    slave[0]  = 0;
}
#endif

